import os
import itertools
from ConfigParser import ConfigParser

from .process import tokenize
from .debug import debug

class BundleError(Exception):
    pass

class Bundle(object):
    def __init__(self, path):
        self.path = path

        cp = ConfigParser()
        cp.read(os.path.join(path, 'activity', 'activity.info'))

        section = 'Activity'

        if cp.has_option(section, 'service_name'):
            self._service_name = cp.get(section, 'service_name')
        else:
            raise BundleError, '%s missing service_name.' % path

        if cp.has_option(section, 'class'):
            self._class = cp.get(section, 'class')

        elif cp.has_option(section, 'exec'):
            self._class = None
            cmdline = cp.get(section, 'exec')
            cmdline = cmdline.replace('$SUGAR_BUNDLE_PATH', self.path)
            cmdline = os.path.expandvars(cmdline)
            self._exec = cmdline
        else:
            raise BundleError, '%s missing exec string.' % path

    def get_argv(self, handle):
        if hasattr(self, '_class') and self._class is not None:
            argv = ['/usr/bin/sugar-activity', self._class]
        else:
            argv = tokenize(self._exec)

        argv += ['-b', self._service_name, '-a', str(handle['activity_id'])]

        if 'object_id' in handle and handle['object_id'] is not None:
            argv += ['-o', str(handle['object_id'])]
        if 'uri' in handle and handle['uri'] is not None:
            argv += ['-u', str(handle['uri'])]
        return argv




def get_bundles(**kwargs):
    """A generator yielding all installed Bundles."""

    # http://standards.freedesktop.org/basedir-spec/basedir-spec-0.6.html...
    search_paths = ['/usr/share/activities',
                    '/usr/local/share/activities',
                    '/home/olpc/Activities']
    files = itertools.chain(*[os.walk(root) for root in search_paths])
    for (dir, dirnames, _) in files:
        try:
            if dir.endswith('.activity'):
                yield Bundle(dir)
            for dirname in dirnames:
                if dirname.endswith('.activity'):
                    yield Bundle(os.path.join(dir, dirname))
        except Exception,e:
            raise e


def find_bundle(service_name):
    for b in get_bundles():
        if b._service_name == service_name:
            return b

    raise KeyError, 'No bundle found for service name: %s' % service_name
